/**
 * votemap plugin for chat (http://www.ganggarrison.com/forums/index.php?topic=33773.0), made by Lorgan3.
 * Adds /votemap, /votenextround and /nextround
 * Create "votableMaps.txt" to set votable maps, if not found the maprotation will be used.
 */

if (!variable_global_exists("commandMap")) {
    global.commandMap = ds_map_create();
}
if (!variable_global_exists("voteCommandMap")) {
    global.voteCommandMap = ds_map_create();
}
if (!variable_global_exists("clientComandMap")) {
    global.clientCommandMap = ds_map_create();
}
if (!variable_global_exists("helpMap")) {
    global.helpMap = ds_map_create();
}

// build a list of votable maps
global.votableMaps = ds_list_create();
if file_exists("votableMaps.txt") {
    var fileHandle, i, mapname;
    fileHandle = file_text_open_read("votableMaps.txt");
    for(i = 1; !file_text_eof(fileHandle); i += 1) {
        mapname = file_text_read_string(fileHandle);
        // remove leading whitespace from the string
        while(string_char_at(mapname, 0) == " " || string_char_at(mapname, 0) == chr(9)) { // while it starts with a space or tab
          mapname = string_delete(mapname, 0, 1); // delete that space or tab
        }
        if(mapname != "" && string_char_at(mapname, 0) != "#") { // if it's not blank and it's not a comment (starting with #)
            ds_list_add(global.votableMaps, mapname);
        }
        file_text_readln(fileHandle);
    }
    file_text_close(fileHandle);
} else {
    ds_list_copy(global.votableMaps, global.map_rotation);
}

ds_map_add(global.commandMap, 'votemap', '
    if (selectedmap || global.chatWindow.alarm[0] > 0) {
        var msg;
        if (selectedmap) {
            msg = "OYou can only vote for the next map once.";
        } else {
            msg = "OYou may not call vote at the moment. Please wait at least " + string(ceil(global.chatWindow.alarm[0] / 30 * global.delta_factor)) + " seconds.";
        }

        if (argument1 == global.myself) {
            execute_string(global.addChatMessage, msg, 255, 0, argument1);
        } else {
            execute_string(global.sendChatMessage, msg, 255, 0, argument1);
        }
    } else {
        ds_list_clear(voteCommand);

        var map, mapList, i, found;
        found = true;
        mapList = ds_list_create();
        ds_list_clear(options);

        if (ds_list_size(argument0) <= 1) {
            map = "";
        } else {
            map = string_lower(ds_list_find_value(argument0, 1));
        }

        for(i = 0; i < ds_list_size(global.votableMaps); i += 1) {
            if (string_count(map, string_lower(ds_list_find_value(global.votableMaps, i))) > 0) {
                ds_list_add(mapList, ds_list_find_value(global.votableMaps, i));
            }
        }

        if (ds_list_size(mapList) == 1) {
            ds_list_add(voteCommand, "votesinglemap");
            ds_list_add(voteCommand, string(ds_list_find_value(mapList, 0)));
            ds_list_add(options, "Yes");
            ds_list_add(options, "No");
        } else if (ds_list_size(mapList) > 1) {
            ds_list_add(voteCommand, "votemap");
            ds_list_shuffle(mapList);
            for(i=0; i < 8 && i < ds_list_size(mapList); i+=1) {
                ds_list_add(voteCommand, string(ds_list_find_value(mapList, i)));
                ds_list_add(options, ds_list_find_value(mapList, i));
            }
        } else if ds_list_size(global.votableMaps) >= 1 {
            if (ds_list_size(argument0) > 1 && map != "") {
                var msg;
                msg = "OSelected map not found, using random selection.";

                if (argument1 == global.myself) {
                    execute_string(global.addChatMessage, msg, 255, 0, argument1);
                } else {
                    execute_string(global.sendChatMessage, msg, 255, 0, argument1);
                }
            }

            ds_list_add(voteCommand, "votemap");
            ds_list_shuffle(global.votableMaps);
            for(i=0; i < 8 && i < ds_list_size(global.votableMaps); i += 1) {
                ds_list_add(voteCommand, string(ds_list_find_value(global.votableMaps, i)));
                ds_list_add(options, ds_list_find_value(global.votableMaps, i));
            }
        } else {
            var msg;
            msg = "ONo maps were found!";

            if (argument1 == global.myself) {
                execute_string(global.addChatMessage, msg, 255, 0, argument1);
            } else {
                execute_string(global.sendChatMessage, msg, 255, 0, argument1);
            }
            found = false;
        }

        if (found && ds_list_size(mapList) == 1) {
            var question, newline;
            if (string_length(string(ds_list_find_value(mapList, 0))) > 12) {
                newline = "#";
            } else {
                newline = "";
            }

            question = "Set next map to " + newline + string(ds_list_find_value(mapList, 0)) + "?";

            execute_string(global.sendVote, 30 * 30, question, options, Player);
        } else if (found) {
            var question;
            question = "Vote for the next map";
            ds_list_add(options, "keep default");

            execute_string(global.sendVote, 30 * 30, question, options, Player);
        }

        ds_list_destroy(mapList);
    }
');
ds_map_add(global.helpMap, 'votemap', 'Starts a vote to set the next map to something different. Use /votemap [Mapname/Map prefix].');

ds_map_add(global.voteCommandMap, 'votemap', '
    var votes, vote, highest;
    for(i=0; i < ds_list_size(argument1); i += 1) {
        votes[i] = 0;
    }
    for(i = 0; i < ds_list_size(argument2); i += 1) {
        votes[ds_list_find_value(argument2, i) - 1] += 1;
    }

    vote = 0;
    highest = votes[0];
    for(i = 1; i < ds_list_size(argument1); i += 1) {
        if (votes[i] > highest) || (votes[i] == highest && random(1) > 0.5) {
            vote = i;
            highest = votes[i];
        }
    }

    if (highest == 0 || vote = ds_list_size(argument0) - 1) {
        var msg;
        msg = "OThe maprotation has not been changed";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, ds_list_size(votes) - 1, global.chatWindow.votePlayer);
    } else {
        selectedmap = true;

        ds_list_clear(fakeMaprotation);
        ds_list_add(fakeMaprotation, ds_list_find_value(argument0, vote + 1));

        global.map_rotation = fakeMaprotation;
        global.currentMapArea = global.totalMapAreas;
        oldMapIndex = GameServer.currentMapIndex;
        GameServer.currentMapIndex = 0;

        var msg;
        msg = "ONext map is " + string(ds_list_find_value(argument0, vote + 1)) + ".";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, vote + 1, global.chatWindow.votePlayer);
    }
');
ds_map_add(global.voteCommandMap, 'votesinglemap', '
    var yes, no, target, i, percent, msg;
    no = 0;
    yes = 0;
    for(i = 0; i < ds_list_size(argument2); i += 1) {
        if (ds_list_find_value(argument2, i) == 1) {
            yes += 1;
        } else {
            no += 1;
        }
    }
    if (no == 0) {
        percent = 100;
    } else if (yes == 0) {
        percent = 0;
    } else {
        percent = floor(yes / (no + yes) * 100);
    }

    if (yes + no >= floor(ds_list_size(global.players) / 2) && percent >= 50) {
        selectedmap = true;

        ds_list_clear(fakeMaprotation);
        ds_list_add(fakeMaprotation, ds_list_find_value(argument0, 1));

        global.map_rotation = fakeMaprotation;
        global.currentMapArea = global.totalMapAreas;
        oldMapIndex = GameServer.currentMapIndex;
        GameServer.currentMapIndex = 0;

        msg = "ONext map is " + string(ds_list_find_value(argument0, 1)) + ".";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, 1, global.chatWindow.votePlayer);
    } else {
        msg = "OThe maprotation has not been changed";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, 0, global.chatWindow.votePlayer);
    }
');

ds_map_add(global.commandMap, 'votenextround', '
    if (global.chatWindow.alarm[0] > 0) {
        msg = "OYou may not call vote at the moment. Please wait at least " + string(ceil(global.chatWindow.alarm[0] / 30 * global.delta_factor)) + " seconds.";

        if (argument1 == global.myself) {
            execute_string(global.addChatMessage, msg, 255, 0, argument1);
        } else {
            execute_string(global.sendChatMessage, msg, 255, 0, argument1);
        }
    } else {
        ds_list_clear(voteCommand);
        ds_list_clear(options);

        ds_list_add(voteCommand, "votenextround");
        var question;
        question = "End the round?";
        ds_list_add(options, "Yes");
        ds_list_add(options, "No");

        execute_string(global.sendVote, 30 * 30, question, options, Player);
    }
');
ds_map_add(global.helpMap, 'votenextround', 'Starts a vote to end the round. Use /votenextround');

ds_map_add(global.voteCommandMap, 'votenextround', '
    var yes, no, i, percent;
    no = 0;
    yes = 0;
    for(i = 0; i < ds_list_size(argument2); i+=1) {
        if (ds_list_find_value(argument2, i) == 1) {
            yes += 1;
        } else {
            no += 1;
        }
    }

    if (no == 0) {
        percent = 100;
    } else if (yes == 0) {
        percent = 0;
    } else  {
        percent = floor(yes / (no + yes) * 100);
    }

    if (yes + no < floor(ds_list_size(global.players) / 2)) {
        var msg;
        msg = "OVoting failed (not enough people voted)";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, 0, global.chatWindow.votePlayer);
    } else if (percent > 50) {
        var msg;
        msg = "OEnding round (" + string(percent) + "% yes)";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, 1, global.chatWindow.votePlayer);

        global.currentMapArea = global.totalMapAreas;
        global.winners = TEAM_SPECTATOR;
    } else {
        var msg;
        msg = "OVoting failed (" + string(100 - percent) + "% no)";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        execute_string(global.sendVoteResults, 2, global.chatWindow.votePlayer);
    }
');

ds_map_add(global.clientCommandMap, 'nextround', '
    if (!global.isHost) {
        var msg;
        msg = "OYou can only use this command as host.";
        execute_string(global.addChatMessage, msg, 255, 0, argument1);
    } else {
        var msg;
        msg = "ORound ended by host.";
        execute_string(global.addChatMessage, msg, 255, 0);
        execute_string(global.sendChatMessage, msg, 255, 0);

        global.currentMapArea = global.totalMapAreas;
        global.winners = TEAM_SPECTATOR;
    }
');
ds_map_add(global.helpMap, 'nextround', 'Command that can only be used by the host to end the round without voting.');
